@@ -1,16 +1,19 @@
-# -*- encoding: utf8 -*-
-# Copyright (C) 2009-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# -*- encoding: utf-8 -*-
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
+
+from __future__ import (absolute_import, division, print_function)
 
 import sys
 from unicodedata import east_asian_width
 
-PY3 = sys.version > '3'
+PY3 = sys.version_info[0] >= 3
 ASCIIONLY = set(chr(c) for c in range(1, 128))
 NARROW = 1
 WIDE = 2
 WIDE_SYMBOLS = set('WF')
 
+
 def uwid(string):
     """Return the width of a string"""
     if not PY3:
@@ -31,9 +34,9 @@ def string_to_charlist(string):
         return list(string)
     result = []
     if PY3:
-        for c in string:
-            result.append(c)
-            if east_asian_width(c) in WIDE_SYMBOLS:
+        for char in string:
+            result.append(char)
+            if east_asian_width(char) in WIDE_SYMBOLS:
                 result.append('')
     else:
         try:
@@ -44,24 +47,22 @@ def string_to_charlist(string):
             string = string.decode('utf-8', 'ignore')
         except UnicodeEncodeError:
             return []
-        for c in string:
-            result.append(c.encode('utf-8'))
-            if east_asian_width(c) in WIDE_SYMBOLS:
+        for char in string:
+            result.append(char.encode('utf-8'))
+            if east_asian_width(char) in WIDE_SYMBOLS:
                 result.append('')
     return result
 
 
-class WideString(object):
+class WideString(object):  # pylint: disable=too-few-public-methods
+
     def __init__(self, string, chars=None):
         try:
             self.string = str(string)
         except UnicodeEncodeError:
             # Here I assume that string is a "unicode" object, because why else
             # would str(string) raise a UnicodeEncodeError?
-            try:
-                self.string = string.encode('latin-1', 'ignore')
-            except:
-                self.string = ""
+            self.string = string.encode('latin-1', 'ignore')
         if chars is None:
             self.chars = string_to_charlist(string)
         else:
@@ -79,8 +80,8 @@ class WideString(object):
         if isinstance(string, str):
             return WideString(self.string + string)
         elif isinstance(string, WideString):
-            return WideString(self.string + string.string,
-                    self.chars + string.chars)
+            return WideString(self.string + string.string, self.chars + string.chars)
+        return None
 
     def __radd__(self, string):
         """
@@ -90,8 +91,8 @@ class WideString(object):
         if isinstance(string, str):
             return WideString(string + self.string)
         elif isinstance(string, WideString):
-            return WideString(string.string + self.string,
-                    string.chars + self.chars)
+            return WideString(string.string + self.string, string.chars + self.chars)
+        return None
 
     def __str__(self):
         return self.string
@@ -99,7 +100,7 @@ class WideString(object):
     def __repr__(self):
         return '<' + self.__class__.__name__ + " '" + self.string + "'>"
 
-    def __getslice__(self, a, z):
+    def __getslice__(self, start, stop):
         """
         >>> WideString("asdf")[1:3]
         <WideString 'sd'>
@@ -122,21 +123,21 @@ class WideString(object):
         >>> WideString("aモ")[0:1]
         <WideString 'a'>
         """
-        if z is None or z > len(self.chars):
-            z = len(self.chars)
-        if z < 0:
-            z = len(self.chars) + z
-        if z < 0:
+        if stop is None or stop > len(self.chars):
+            stop = len(self.chars)
+        if stop < 0:
+            stop = len(self.chars) + stop
+        if stop < 0:
             return WideString("")
-        if a is None or a < 0:
-            a = 0
-        if z < len(self.chars) and self.chars[z] == '':
-            if self.chars[a] == '':
-                return WideString(' ' + ''.join(self.chars[a:z - 1]) + ' ')
-            return WideString(''.join(self.chars[a:z - 1]) + ' ')
-        if self.chars[a] == '':
-            return WideString(' ' + ''.join(self.chars[a:z - 1]))
-        return WideString(''.join(self.chars[a:z]))
+        if start is None or start < 0:
+            start = 0
+        if stop < len(self.chars) and self.chars[stop] == '':
+            if self.chars[start] == '':
+                return WideString(' ' + ''.join(self.chars[start:stop - 1]) + ' ')
+            return WideString(''.join(self.chars[start:stop - 1]) + ' ')
+        if self.chars[start] == '':
+            return WideString(' ' + ''.join(self.chars[start:stop - 1]))
+        return WideString(''.join(self.chars[start:stop]))
 
     def __getitem__(self, i):
         """
@@ -149,7 +150,7 @@ class WideString(object):
         """
         if isinstance(i, slice):
             return self.__getslice__(i.start, i.stop)
-        return self.__getslice__(i, i+1)
+        return self.__getslice__(i, i + 1)
 
     def __len__(self):
         """
